@@ -14,20 +14,12 @@ workers = function(n_jobs, data=NULL, reuse=TRUE, template=list(), log_worker=FA
                    qsys_id=getOption("clustermq.scheduler", qsys_default),
                    verbose=FALSE, ...) {
     if (n_jobs == 0)
-        return(get("LOCAL", envir=parent.env(environment()))$new())
+        qsys_id = "LOCAL"
 
-    gc() # be sure to clean up old rzmq handles (zeromq/libzmq/issues/1108)
+    gc() # be sure to clean up old zmq handles (zeromq/libzmq/issues/1108)
     qsys = get(toupper(qsys_id), envir=parent.env(environment()))
     qsys = qsys$new(data=data, reuse=reuse, ...)
-
-    if (log_worker && is.null(template$log_file)) {
-        .Deprecated(msg="'log_worker' is deprecated, use template=list(log_file=...) instead")
-        template$log_file = paste0("cmq", qsys$id, ".log")
-    }
-
-    template$n_jobs = n_jobs
-    if (verbose)
-        message("Submitting ", n_jobs, " worker jobs (ID: ", qsys$id, ") ...")
-    do.call(qsys$submit_jobs, template)
+    args = list(n_jobs=n_jobs, log_worker=log_worker, verbose=verbose)
+    do.call(qsys$submit_jobs, c(template, args))
     qsys
 }
